@@ -1,11 +1,11 @@
 from typing import List
 from functools import reduce
-import re
 from hestia_earth.schema import SiteSiteType
 
 from hestia_earth.validation.geojson import get_geojson_area
 from hestia_earth.validation.gadm import exec_request
-from .shared import list_has_props, validate_dates, validate_list_dates, validate_list_duplicated, diff_in_years
+from .shared import list_has_props, validate_dates, validate_list_dates, validate_list_duplicates, diff_in_years, \
+    validate_list_min_max, validate_region, validate_country
 
 
 SOIL_TEXTURE_IDS = ['sandContent', 'siltContent', 'clayContent']
@@ -47,8 +47,8 @@ def validate_soilTexture(measurements: List[dict]):
 
 def validate_depths(measurements: List[dict]):
     def validate(values):
-        measurement = values[1]
         index = values[0]
+        measurement = values[1]
         return measurement['depthUpper'] < measurement['depthLower'] or {
             'level': 'error',
             'dataPath': f".measurements[{index}].depthLower",
@@ -59,6 +59,34 @@ def validate_depths(measurements: List[dict]):
     return next((x for x in results if x is not True), True)
 
 
+def value_range_error(value: int, minimum: int, maximum: int):
+    return 'minimum' if minimum is not None and value < minimum else \
+        'maximum' if maximum is not None and value > maximum else False
+
+
+def validate_measurements_value(measurements: List[dict]):
+    def validate(values):
+        index = values[0]
+        measurement = values[1]
+        props = measurement.get('term', {}).get('defaultProperties', [])
+        mininum = next((prop.get('value') for prop in props if prop.get('term', {}).get('@id') == 'minimum'), None)
+        maximum = next((prop.get('value') for prop in props if prop.get('term', {}).get('@id') == 'maximum'), None)
+        value = measurement.get('value')
+        error = value_range_error(value, mininum, maximum) if value is not None else False
+        return error is False or ({
+            'level': 'error',
+            'dataPath': f".measurements[{index}].value",
+            'message': f"should be above {mininum}"
+        } if error == 'minimum' else {
+            'level': 'error',
+            'dataPath': f".measurements[{index}].value",
+            'message': f"should be below {maximum}"
+        })
+
+    results = list(map(validate, enumerate(measurements)))
+    return next((x for x in results if x is not True), True)
+
+
 def validate_lifespan(infrastructure: List[dict]):
     def validate(values):
         value = values[1]
@@ -96,28 +124,6 @@ def validate_area(site: dict):
         return True
 
 
-def validate_region(site: dict):
-    country = site.get('country')
-    region_id = site.get('region').get('@id')
-    return region_id[0:8] == country.get('@id') or {
-        'level': 'error',
-        'dataPath': '.region',
-        'message': 'must be within the country',
-        'params': {
-            'country': country.get('name')
-        }
-    }
-
-
-def validate_country(site: dict):
-    country_id = site.get('country').get('@id')
-    return bool(re.search(r'GADM-[A-Z]{3}', country_id)) or {
-        'level': 'error',
-        'dataPath': '.country',
-        'message': 'must be a country'
-    }
-
-
 def need_validate_coordinates(site: dict):
     return 'latitude' in site and 'longitude' in site and site.get('siteType') in INLAND_TYPES
 
@@ -125,7 +131,7 @@ def need_validate_coordinates(site: dict):
 def validate_coordinates(site: dict):
     latitude = site.get('latitude')
     longitude = site.get('longitude')
-    country = site.get('country')
+    country = site.get('country', {})
     region = site.get('region')
     gadm_id = region.get('@id') if region else country.get('@id')
     id = None if region else gadm_id  # pass in the id for a country for faster results
@@ -147,9 +153,11 @@ def validate_site(site: dict):
         validate_area(site) if 'area' in site and 'boundary' in site else True
     ] + ([
         validate_list_dates(site, 'measurements'),
-        validate_soilTexture(site['measurements']),
-        validate_depths(site['measurements']),
-        validate_list_duplicated(site, 'measurements', [
+        validate_list_min_max(site, 'measurements'),
+        validate_soilTexture(site.get('measurements')),
+        validate_depths(site.get('measurements')),
+        validate_measurements_value(site.get('measurements')),
+        validate_list_duplicates(site, 'measurements', [
             'term.@id',
             'method.@id',
             'methodDescription',
@@ -160,5 +168,5 @@ def validate_site(site: dict):
         ])
     ] if 'measurements' in site else []) + ([
         validate_list_dates(site, 'infrastructure'),
-        validate_lifespan(site['infrastructure'])
+        validate_lifespan(site.get('infrastructure'))
     ] if 'infrastructure' in site else [])
